// *********************(c)*2004*********************>
// -holzilib--holzilib--holzilib--holzilib--holzilib->
// ++++PD-External++by+Georg+Holzmann++grh@gmx.at++++>
//
// PDContainer: 
// this is a port of the containers from the C++ STL
// (Standard Template Library)
// for usage see the documentation and PD help files
// for license see readme.txt
//
// TMapBase.cpp

#include "include/TMapBase.h"


//----------------------------------------------------
/* adds a key-value pair
 */
TemplateDefinition  // see GlobalStuff.h
void TMapBase<ContainerType,ContTypeIterator>::add(atom key, atom value)
{
  Element selector;
  selector.h_namespace = h_namespace_;
  selector.data = key;

  data_[selector] = value;
}

//----------------------------------------------------
/* gets a value to the specific key
 */
TemplateDefinition  // see GlobalStuff.h
atom &TMapBase<ContainerType,ContTypeIterator>::get(atom &key) const
{
  Element selector;
  selector.h_namespace = h_namespace_;
  selector.data = key;

  return data_[selector];
}

//----------------------------------------------------
/* prints all the data of the current namespace to the console
 */
TemplateDefinition  // see GlobalStuff.h
void TMapBase<ContainerType,ContTypeIterator>::printAll()
{
  ContTypeIterator iter  = data_.begin();

  post("\nh_map: printing namespace %s",h_namespace_.c_str());
  post("--------------------------------------------------");

  int count = 0;
  while(iter != data_.end())
    {
      if((*iter).first.h_namespace == h_namespace_)
	{
	  ostringstream output(" ");

	  if ((*iter).first.data.a_type == A_FLOAT)
	    output << "float " << (*iter).first.data.a_w.w_float;
	  if ((*iter).first.data.a_type == A_SYMBOL)
	    output << "symbol " << (*iter).first.data.a_w.w_symbol->s_name;

	  output << "  -  ";

	  if ((*iter).second.a_type == A_FLOAT)
	    output << "float " << (*iter).second.a_w.w_float;
	  if ((*iter).second.a_type == A_SYMBOL)
	    output << "symbol " << (*iter).second.a_w.w_symbol->s_name;

	  post("%s",output.str().c_str());
	  count = 1;
	}
      iter++;
    }
  if(!count)
    post("no data in current namespace!");
  post("--------------------------------------------------");
}

//----------------------------------------------------
/* saves all the data of the current namespace to a file
 * Fileformat: <key_type> <key> <data_type> <data>
 * e.g.:
 * s foo f 12.34234
 * s foo s hallo
 * type: s=symbol, f=float
 * ATTENTION: if the file exists, all the old data of
 * the file is lost
 * returns true on success
 */
TemplateDefinition  // see GlobalStuff.h
bool TMapBase<ContainerType,ContTypeIterator>::saveToFile(string filename)
{
  ofstream outfile;
  ContTypeIterator iter  = data_.begin();

  outfile.open(filename.c_str());

  if(!outfile)
    return false;

  while(iter != data_.end())
    {
      if((*iter).first.h_namespace == h_namespace_)
	{
	  if ((*iter).first.data.a_type == A_FLOAT)
	    outfile << "f " << (*iter).first.data.a_w.w_float << " ";
	  if ((*iter).first.data.a_type == A_SYMBOL)
	    outfile << "s " << (*iter).first.data.a_w.w_symbol->s_name << " ";

	  if ((*iter).second.a_type == A_FLOAT)
	    outfile << "f " << (*iter).second.a_w.w_float << endl;
	  if ((*iter).second.a_type == A_SYMBOL)
	    outfile << "s " << (*iter).second.a_w.w_symbol->s_name << endl;
	}
      iter++;
    }

  outfile.close();

  return true;
}

//----------------------------------------------------
/* Fileformat: <key_type> <key> <data_type> <data>
 * e.g.:
 * s foo f 12.34234
 * s foo s hallo
 * type: s=symbol, f=float
 * returns true on success
 */
TemplateDefinition  // see GlobalStuff.h
bool TMapBase<ContainerType,ContTypeIterator>::readFromFile(string filename)
{
  ifstream infile;
  infile.open(filename.c_str());

  if(!infile)
      return false;

  string line;
  atom dummy;
  char type;
  string symbol;
  t_float number;
  Element selector;
  selector.h_namespace = h_namespace_;

  while (getline(infile, line))
    {
      istringstream instream(line);

      instream >> type;
      if (type == 's')
	{
	  instream >> symbol;
	  SETSYMBOL(&dummy,gensym(const_cast<char*>(symbol.c_str())));
	  selector.data = dummy;
	}
      if (type == 'f')
	{
	  instream >> number;
	  SETFLOAT(&dummy,number);
	  selector.data = dummy;
	}

      instream >> type;
      if (type == 's')
	{
	  instream >> symbol;
	  SETSYMBOL(&dummy,gensym(const_cast<char*>(symbol.c_str())));
	  data_.insert(std::pair<Element,atom>(selector,dummy));
	}
      if (type == 'f')
	{
	  instream >> number;
	  SETFLOAT(&dummy,number);
	  data_.insert(std::pair<Element,atom>(selector,dummy));
	}
    }

  infile.close();

  return true;
}
