// *********************(c)*2004*********************>
// -holzilib--holzilib--holzilib--holzilib--holzilib->
// ++++PD-External++by+Georg+Holzmann++grh@gmx.at++++>
//
// PDContainer: 
// this is a port of the containers from the C++ STL
// (Standard Template Library)
// for usage see the documentation and PD help files
// for license see readme.txt
//
// TPDContainer.cpp

#include "include/TPDContainer.h" 


//---------------------------------------------------
/* clears all the data of all namespaces
 * off all objects from the same container
 */
TemplateDefinition  // see GlobalStuff.h
void TPDContainer<ContainerType,ContTypeIterator>::clearAll()
{
  data_.clear();
}

//---------------------------------------------------
/* clears all the data of the current namespaces
 * in all objects from the same container
 */
TemplateDefinition  // see GlobalStuff.h
void TPDContainer<ContainerType,ContTypeIterator>::clearNamespace()
{
  // TODO !!!!!!!
}

//---------------------------------------------------
/* prints all the data of the current namespace to the console
 */
TemplateDefinition  // see GlobalStuff.h
void TPDContainer<ContainerType,ContTypeIterator>::printAll()
{
  ContTypeIterator iter = data_.begin();

  post("\n%s: printing namespace %s",dataname_,h_namespace_.c_str());
  post("--------------------------------------------------");

  int count = 0;
  while(iter != data_.end())
    {
      if((*iter).h_namespace == h_namespace_)
	{
	  ostringstream output(" ");

	  if ((*iter).data.a_type == A_FLOAT)
	    output << "float " << (*iter).data.a_w.w_float;
	  if ((*iter).data.a_type == A_SYMBOL)
	    output << "symbol " << (*iter).data.a_w.w_symbol->s_name;

	  post("%s",output.str().c_str());
	  count = 1;
	}
      iter++;
    }
  if(!count)
    post("no data in current namespace!");
  post("--------------------------------------------------");
}

//---------------------------------------------------
/* saves all the data of the current namespace to a file
 * Fileformat: <data_type> <data>
 * e.g.:
 * f 12.34234
 * s foo
 * type: s=symbol, f=float
 * ATTENTION: if the file exists, all the old data of
 * the file is lost
 * ATTENTION: some containers (e.g. map) have other
 * fileformats !
 * returns true on success
 */
TemplateDefinition  // see GlobalStuff.h
bool TPDContainer<ContainerType,ContTypeIterator>::saveToFile(string filename)
{
  ofstream outfile;
  ContTypeIterator iter = data_.begin();

  outfile.open(filename.c_str(),std::ios::trunc);

  if(!outfile)
    return false;

  while(iter != data_.end())
    {
      if((*iter).h_namespace == h_namespace_)
	{
	  if ((*iter).data.a_type == A_FLOAT)
	    outfile << "f " << (*iter).data.a_w.w_float << endl;
	  if ((*iter).data.a_type == A_SYMBOL)
	    outfile << "s " << (*iter).data.a_w.w_symbol->s_name << endl;
	}
      iter++;
    }

  outfile.close();

  return true;
}

//---------------------------------------------------
/* adds the data of the file to the current namespace
 * Fileformat: <data_type> <data>
 * e.g.:
 * f 12.34234
 * s foo
 * type: s=symbol, f=float
 * ATTENTION: some containers (e.g. map) have other
 * fileformats !
 * returns true on success
 */
TemplateDefinition  // see GlobalStuff.h
bool TPDContainer<ContainerType,ContTypeIterator>::readFromFile(string filename)
{
  ifstream infile;
  infile.open(filename.c_str());

  if(!infile)
      return false;

  string line;
  atom dummy;
  char type;
  string symbol;
  t_float number;
  Element selector;
  selector.h_namespace = h_namespace_;

  while (getline(infile, line))
    {
      istringstream instream(line);

      instream >> type;
      if (type == 's')
	{
	  instream >> symbol;
	  SETSYMBOL(&dummy,gensym(const_cast<char*>(symbol.c_str())));
	  selector.data = dummy;
	  data_.insert(selector);
	}
      if (type == 'f')
	{
	  instream >> number;
	  SETFLOAT(&dummy,number);
	  selector.data = dummy;
	  data_.insert(selector);
	}
    }

  infile.close();

  return true;
}
